@@ -1,39 +1,94 @@
-#' Assist calibration of DYRESM-CAEDYM model. Before using this function, make sure that you have set up "Bin" and "Files" sub-folders under the DYCD model folder.
+#' Assist calibration of DYRESM-CAEDYM model.
 #'
-#' @description This function tries different combinations of selected parameter values and
-#'   outputs corresponding values of fit-of-goodness by calculating some objective functions.
-#'   Then users can choose the optimal set of parameter values to calibrate the model.
+#' @description This function carries out simulations with a large number
+#' of possible combinations of parameter values that users regard as
+#' potentially suitable for their model calibration, and calculates
+#' the values of nominated objective functions (i.e., statistical measures
+#' of goodness of fit) for each combination. Based on the calculated
+#' objective function values, users can determine the optimal set(s)
+#' of parameter values or narrow the ranges of possible parameter values.
 #'
-#' @param cal.para a data frame or a character string naming an external .csv file where below columns are included:
-#'                 "Parameter" where parameter names (abbreviation is allowed),
-#'                 "Min", "Max", and "Increment" describing the minimum and maximum parameter values and expected increment in the value range,
-#'                 "Input_file" and "Line_NO" listing in which configuration file at which line can the parameter can be found.
-#' @param combination a vector of string character of how to pick up combinations of parameter values."random" - the function randomly
-#'    picks up given number of combinations; "all" - the function tries all possible combinations of parameter values.
-#' @param n the number of randomly selections. Must be provided if combination = "random".
-#' @param model.var a vector of string character of modelled variables for calibration.
-#'       If the chlorophyll of multiple phytoplankton groups is used collectively for calibration, use "CHLA" and further specify which phytoplankton groups are to be combined in the argument of "phyto.group".
-#'       If phytoplankton groups are separately calibrated, list their abbreviation in this argument. Five abbreviations are supported: CHLOR, FDIAT, NODUL, CYANO and CRYPT.
-#' @param phyto.group a vector of simulated phytoplankton groups, including CHLOR, FDIAT, NODUL, CYANO and CRYPT.
-#' @param obs.data a character string naming a file of observed lake data. This file needs to be prepared in a given format (see example data).
-#' @param objective.function a vector of string character claiming what objective function(s) to be used for calibration.
-#' Selected from the following five functions: "NSE": Nash-Sutcliffe efficiency coefficient, "RMSE": Root Mean Square Error,
-#'    "MAE": Mean Absolute Error, "RAE": Relative Absolute Error, "Pearson": Pearson's r.
-#' @param start.date,end.date the beginning and ending simulation dates for the intended DYRESM-CAEDYM calibration. The date format must be "\%Y-\%m-\%d".
-#' @param dycd.wd the directory where input files (including the bat file) to DYRESM-CAEDYM are stored. either relative or absolute path is allowed.
-#' @param dycd.output a character string naming the output file of the model calibration.
-#' @param file.name a character string naming a .csv file for writing out the auto-calibration results.
-#' @param verbose if TRUE, the auto-calibration information is printed.
-#' @param parallel if TRUE, the calibration process can be run on multiple cores.
-#' @param n.cores When parallel is TRUE, n.cores is the number of cores the calibration function will be run on. If not provided,
-#'        the default value is the number of available cores on the computer -1.
-#' @param write.out if TRUE, the auto-calibration results are saved a file with a file name set by the "file.name" argument.
+#' @param cal.para a data frame or a character string naming an external
+#' .csv file where below column names are mandatory: "Parameter" describing
+#' parameter names (abbreviation is allowed), "Min", "Max", and
+#' "Increment" describing the minimum and maximum parameter values
+#' and expected increment in the value range, "Input_file" and
+#' "Line_NO" listing in which configuration file at which line
+#' the parameter can be found.
+#'
+#' @param combination a vector of string character of how to pick up
+#'  combinations of parameter values. "random" - the function randomly
+#'  picks up a given number of combinations; "all" - the function tries
+#'  all possible combinations of parameter values.
+#'
+#' @param n the number of random selections.
+#' Must be provided if combination = "random".
+#'
+#' @param model.var a vector of string character of modelled
+#' variables for calibration. the character should be from the
+#' 'var.name' column of 'data(output_name)'.
+#' Note that if model calibration needs to regard chlorophyll
+#' of multiple phytoplankton groups as a whole,
+#' model.var should use "CHLA" and individual phytoplankton group
+#' should be specified through the "phyto.group" argument.
+#' If phytoplankton groups are separately calibrated,
+#' simply list their character in this argument (model.var).
+#'
+#' @param phyto.group a vector of simulated phytoplankton groups,
+#' including CHLOR, FDIAT, NODUL, CYANO and CRYPT.
+#'
+#' @param obs.data a data frame or a character string naming a csv file of
+#' observed lake data.
+#' The observed lake data need to include below columns:
+#'  1) 'Date' in format of "\%Y-\%m-\%d"
+#'  2) 'Depth' (integer)
+#'  3) Water quality variables (use string characters of model var
+#'     as column names).
+#'  see example data 'data(obs_temp)'.
+#'
+#' @param objective.function a vector of string character describing which
+#'  objective function(s) to be used for calibration. Selected from
+#'  the following five functions:
+#'  "NSE": Nash-Sutcliffe efficiency coefficient,
+#'  "RMSE": Root Mean Square Error,
+#'  "MAE": Mean Absolute Error,
+#'  "RAE": Relative Absolute Error,
+#'  "Pearson": Pearson's r.
+#'
+#' @param start.date,end.date the beginning and end simulation dates
+#'  for the intended DYRESM-CAEDYM calibration.
+#'  The date format must be "\%Y-\%m-\%d".
+#'  The two dates should be consistent with model configurations.
+#'
+#' @param dycd.wd the directory where input files (including the bat file)
+#'  to DYRESM-CAEDYM are stored.
+#'
+#' @param dycd.output a character string naming the output file of
+#'  model simulation.
+#'
+#' @param file.name a character string naming a .csv file where
+#'  the results of this function are written to. Needed if 'write.out'
+#'  = TRUE.
+#'
+#' @param verbose if TRUE, model calibration information is printed.
+#'
+#' @param parallel if TRUE, the calibration process is run on multiple cores.
+#'
+#' @param n.cores When 'parallel' is TRUE, n.cores is the number of cores
+#'  the calibration function will be run on. If not provided,
+#'  the default value is the number of available cores on the computer -1.
+#'
+#' @param write.out if TRUE, model calibration results are saved in a file
+#'  with a file name set by the "file.name" argument.
 #'
 #' @import dplyr
 #' @importFrom utils read.csv write.csv
-#' @return a dataframe of trialed values of parameters and corresponding values of objective function(s).
+#' @return a data frame of all tested values of parameters and
+#'  corresponding values of the objective function(s).
 #'
-#' @note No executable examples are provided to illustrate the use of this function, as this function relies on the DYRESM-CAEDYM executables to work.
+#' @note No executable examples are provided to illustrate the
+#'  use of this function, as this function relies on the
+#'  DYRESM-CAEDYM executables to work.
 #'
 #' @export
 
@@ -64,9 +119,10 @@ calib_assist<-function(cal.para,
     para.raw <- read.csv(file = cal.para)
   }
 
-  seq.list<-list()
-  for(i in 1:nrow(para.raw)){
-    seq.list[[i]]<-seq(para.raw$Min[i],para.raw$Max[i],length.out =para.raw$N_values[i])
+  seq.list <- list()
+  for(i in seq_len(nrow(para.raw))){
+    seq.list[[i]] <- seq(para.raw$Min[i],para.raw$Max[i],
+                       length.out =para.raw$N_values[i])
   }
 
   if(length(seq.list)==1){
@@ -90,11 +146,11 @@ calib_assist<-function(cal.para,
   # 2.all or random - number of iterations
   #---
   if(combination =="all"){
-    iteration <- 1:nrow(para.df)
+    iteration <- seq_len(nrow(para.df))
     para.df <- para.df[iteration, ]
   }
   if(combination =="random"){
-    iteration <- base::sample(x = c(1:nrow(para.df)), size = n)
+    iteration <- base::sample(x = seq_len(nrow(para.df)), size = n)
     para.df <- para.df[iteration, ]
   }
 
@@ -108,14 +164,20 @@ calib_assist<-function(cal.para,
   #---
   # 4. read observed lake data for calculating objective functions
   #---
-  obs.lake <- read.csv(obs.data)
-  obs.lake$Date <- as.Date(obs.lake$Date, format="%d/%m/%Y")
+  if(is.data.frame(obs.data)){
+    obs.lake <- obs.data
+  }else{
+    obs.lake <- read.csv(obs.data)
+    obs.lake$Date <- as.Date(obs.lake$Date, format = "%Y-%m-%d")
+  }
+
   obs.lake <- obs.lake %>%
     filter(Date >= sim.date[1] & Date <= sim.date[length(sim.date)])
 
   if(!all(model.var %in% colnames(obs.lake))){
-    cat(model.var[!(model.var %in% colnames(obs.lake))], "are not provided for observed data,
-      so that these variables will be excluded for model calibration.")
+    cat(model.var[!(model.var %in% colnames(obs.lake))],
+    "are not provided for observed data,
+    so that these variables will be excluded for model calibration.")
   }
   actual.model.var <- model.var[model.var %in% colnames(obs.lake)]
   obs.list <- lapply(match(actual.model.var, colnames(obs.lake)),
@@ -146,57 +208,75 @@ calib_assist<-function(cal.para,
     }
 
     # allocate iterations to the available cores
-    sim.cores <- rep(1:n.cores, length.out=length(iteration))
+    sim.cores <- rep(seq_len(n.cores), length.out=length(iteration))
 
-    # find the files that are needed/wanted to run this model (par, bio, chm, sed files will copy later)
-    files.model <- list.files(path = dycd.wd, pattern = '.*\\.(bat|cfg|con|inf|int|met|pro|stg|wdr)$',
-                              recursive = FALSE, include.dirs = TRUE, full.names = TRUE)
+    # find the files that are needed/wanted to run this model
+    # (par, bio, chm, sed files will copy later)
+    files.model <-
+      list.files(path = dycd.wd,
+                 pattern = '.*\\.(bat|cfg|con|inf|int|met|pro|stg|wdr)$',
+                 recursive = FALSE, include.dirs = TRUE,
+                 full.names = TRUE)
 
-    # create sacrificial folders with a copy of the model for each core being used
-    for (c in 1:n.cores) {
+    # create sacrificial folders with a copy of the model for
+    # each core being used
+    for (c in seq_len(n.cores)) {
       dir.core <- paste0(dycd.wd,'/core',c)
       dir.create(dir.core)
-      file.copy(files.model, to = dir.core, recursive = T, overwrite = T)
-      R.utils::copyDirectory(paste0(dycd.wd, "/Bin"), to = paste0(dir.core, "/Bin"), recursive = T, overwrite = T)
-      #R.utils::copyDirectory(paste0(dycd.wd, "/Files"), to = paste0(dir.core, "/Files"), recursive = T, overwrite = T)  # CAEDYM
+      file.copy(files.model, to = dir.core, recursive = TRUE, overwrite = TRUE)
+      R.utils::copyDirectory(paste0(dycd.wd, "/Bin"),
+                             to = paste0(dir.core, "/Bin"),
+                             recursive = TRUE, overwrite = TRUE)
+      #R.utils::copyDirectory(paste0(dycd.wd, "/Files"),
+      # to = paste0(dir.core, "/Files"),
+      # recursive = TRUE, overwrite = TRUE)  # CAEDYM
     }
 
     ##### initiate and execute the cluster run
     cl <- parallel::makeCluster(n.cores)
 
-    # export any necessary objects and/or functions to the cluster before running
+    # export any necessary objects and/or functions to the cluster
+    # before running
     parallel::clusterEvalQ(cl, library("dycdtools"))
-    parallel::clusterExport(cl, varlist = c("dycd.wd","dir.output","sim.cores","para.df","para.raw","iteration"),envir = environment())
+    parallel::clusterExport(cl, varlist = c("dycd.wd","dir.output","sim.cores",
+                                            "para.df","para.raw","iteration"),
+                            envir = environment())
 
-    parallel::clusterApply(cl, 1:length(iteration), run_iteration, dycd.wd)
+    parallel::clusterApply(cl, seq_len(length(iteration)),
+                           run_iteration, dycd.wd)
 
     # clean up cluster
     try({parallel::stopCluster(cl)})
 
     ##### clean up the created folders
-    unlink(paste0(dycd.wd, "/core", seq(1:n.cores)), recursive = T)
+    unlink(paste0(dycd.wd, "/core", seq_len(n.cores)), recursive = TRUE)
 
     #---
     # calculate objective function values
     #---
-    for(b in 1:length(iteration)){
-      var.values <- ext_output(dycd.output = paste0(dir.output, "/DYsim_", b, ".nc"),
+    for(b in seq_along(iteration)) {
+      var.values <- ext_output(dycd.output = paste0(dir.output,
+                                                    "/DYsim_", b, ".nc"),
                                var.extract = actual.model.var)
 
       if("CHLA" %in% model.var){
         actual.model.var.2 <- append(actual.model.var, phyto.group)
-        actual.model.var.2 <- actual.model.var.2[-which(actual.model.var.2 == "CHLA")]
+        actual.model.var.2 <- actual.model.var.2[-which(actual.model.var.2 ==
+                                                          "CHLA")]
 
         var.values <- ext_output(dycd.output = dycd.output,
                                  var.extract = actual.model.var.2)
       }
 
-      for(nm in 1:length(var.values)){
-        expres <- paste0(names(var.values)[nm], "<-data.frame(var.values[[", nm, "]])")
+      for(nm in seq_along(var.values)){
+        expres <- paste0(names(var.values)[nm],
+                         "<-data.frame(var.values[[", nm, "]])")
         eval(parse(text = expres))
       }
 
-      lake.depth.list <- apply(dyresmLAYER_HTS_Var, 2, FUN = function(a) hgt_to_dpt(a[!is.na(a)]))
+      lake.depth.list <- apply(dyresmLAYER_HTS_Var, 2,
+                               FUN = function(a) hgt_to_dpt(a[!is.na(a)]))
+
       max.depth <- ceiling(max(unlist(lake.depth.list)))
 
       for (var in actual.model.var){
@@ -204,21 +284,29 @@ calib_assist<-function(cal.para,
 
         if(var == "CHLA"){
           for(phyto in phyto.group){
-            expres <- paste0("sim.", phyto, "<-", output_name$output.name[match(phyto, output_name$var.name)])
+            expres <-
+              paste0("sim.", phyto, "<-",
+                     output_name$output.name[match(phyto,output_name$var.name)])
             eval(parse(text = expres))
           }
-          expres <- paste0("sim.var<-", paste0("sim.", phyto.group, collapse = "+"))
+          expres <- paste0("sim.var<-",
+                           paste0("sim.", phyto.group, collapse = "+"))
           eval(parse(text = expres))
         }
 
         if(var != "CHLA"){
-          expres <- paste0("sim.var<-", output_name$output.name[match(var, output_name$var.name)])
+          expres <- paste0("sim.var<-",
+                           output_name$output.name[match(var,
+                                                         output_name$var.name)])
           eval(parse(text = expres))
         }
 
-        try.return <- try(interpolated <- interpol(layerHeights = dyresmLAYER_HTS_Var,
-                                                   var = sim.var,
-                                                   min.depth = 0, max.depth = max.depth, by.value = 0.5))
+        try.return <-
+          try(interpolated <- interpol(layerHeights = dyresmLAYER_HTS_Var,
+                                       var = sim.var,
+                                       min.depth = 0,
+                                       max.depth = max.depth,
+                                       by.value = 0.5))
 
         if(class(try.return)[1]!="try-error"){
 
@@ -227,7 +315,8 @@ calib_assist<-function(cal.para,
                                      fun = objective.function,
                                      start.date,
                                      end.date,
-                                     min.depth = 0, max.depth = max.depth, by.value = 0.5)
+                                     min.depth = 0, max.depth = max.depth,
+                                     by.value = 0.5)
 
           if(exists("NSE.list")){
             NSE.list[[var]][b] <- obj.value['NSE']
@@ -301,7 +390,7 @@ calib_assist<-function(cal.para,
 
   if(!parallel){
 
-    for(i in 1:length(iteration)){
+    for(i in seq_along(iteration)){
 
       if(verbose){
         cat(i, "/", length(iteration), "\n")
@@ -310,8 +399,9 @@ calib_assist<-function(cal.para,
       #---
       # change the parameter values in the input files
       #---
-      for(m in 1:ncol(para.df)){
-        change_input_file(input_file = paste0(dycd.wd, "/", para.raw$Input_file[m]),
+      for(m in seq_len(ncol(para.df))){
+        change_input_file(input_file = paste0(dycd.wd, "/",
+                                              para.raw$Input_file[m]),
                           row_no = para.raw$Line_NO[m],
                           new_value = para.df[i, m])
       }
@@ -335,18 +425,21 @@ calib_assist<-function(cal.para,
 
       if("CHLA" %in% model.var){
         actual.model.var.2 <- append(actual.model.var, phyto.group)
-        actual.model.var.2 <- actual.model.var.2[-which(actual.model.var.2 == "CHLA")]
+        actual.model.var.2 <-
+          actual.model.var.2[-which(actual.model.var.2 == "CHLA")]
 
         var.values <- ext_output(dycd.output = dycd.output,
                                  var.extract = actual.model.var.2)
       }
 
-      for(nm in 1:length(var.values)){
-        expres <- paste0(names(var.values)[nm], "<- data.frame(var.values[[", nm ,"]])")
+      for(nm in seq_along(var.values)){
+        expres <- paste0(names(var.values)[nm],
+                         "<- data.frame(var.values[[", nm ,"]])")
         eval(parse(text = expres))
       }
 
-      lake.depth.list <- apply(dyresmLAYER_HTS_Var, 2, FUN = function(a) hgt_to_dpt(a[!is.na(a)]))
+      lake.depth.list <- apply(dyresmLAYER_HTS_Var, 2,
+                               FUN = function(a) hgt_to_dpt(a[!is.na(a)]))
       max.depth <- ceiling(max(unlist(lake.depth.list)))
 
       for (var in actual.model.var){
@@ -354,21 +447,29 @@ calib_assist<-function(cal.para,
 
         if(var == "CHLA"){
           for(phyto in phyto.group){
-            expres <- paste0("sim.", phyto, "<-", output_name$output.name[match(phyto, output_name$var.name)])
+            expres <-
+              paste0("sim.", phyto, "<-",
+                     output_name$output.name[match(phyto,
+                                                   output_name$var.name)])
             eval(parse(text = expres))
           }
-          expres <- paste0("sim.var <- ", paste0("sim.", phyto.group, collapse = "+"))
+          expres <- paste0("sim.var <- ",
+                           paste0("sim.", phyto.group, collapse = "+"))
           eval(parse(text = expres))
         }
 
         if(var != "CHLA"){
-          expres <- paste0("sim.var <- ", output_name$output.name[match(var, output_name$var.name)])
+          expres <- paste0("sim.var <- ",
+                           output_name$output.name[match(var,
+                                                         output_name$var.name)])
           eval(parse(text = expres))
         }
 
-        try.return <- try(interpolated <- interpol(layerHeights = dyresmLAYER_HTS_Var,
-                                                   var = sim.var,
-                                                   min.depth = 0, max.depth = max.depth, by.value = 0.5))
+        try.return <- try(interpolated <-
+                            interpol(layerHeights = dyresmLAYER_HTS_Var,
+                                     var = sim.var,
+                                     min.depth = 0,
+                                     max.depth = max.depth, by.value = 0.5))
 
         if(class(try.return)[1] != "try-error"){
 
