@@ -1,22 +1,38 @@
-#' Profile plot of simulated variable values vs. depth
-#'
-#' @description
 #' A post-processing function used to visualise model output in a profile graph.
 #'
-#' @param sim interpolated values of variable.
-#' @param obs observed values of variable.
-#' @param sim.start,sim.end the beginning and ending simulation dates for the intended DYRESM-CAEDYM model run. The date format must be "\%Y-\%m-\%d".
-#' @param plot.start,plot.end the beginning and ending dates for the plotting purpose. The date format must be "\%Y-\%m-\%d".
-#' @param xlabel the x axis label of the profile figure
-#' @param min.depth,max.depth,by.value minimum and maximum depth for the profile plot at the depth increment of by.value.
+#' @description Profile plot shows vertical profiles of simulation outputs and
+#'  corresponding observations for all dates where observations are available.
+#'
+#' @param sim a matrix of simulated variables. This matrix can be generated
+#' by running the "interpol" function.
+#'
+#' @param obs a data frame having three columns to describe observed values of
+#'  a water quality variable. These three columns are 'Date' (as '\%Y-\%m-\%d'),
+#'  'Depth', and the designated variable name which can be found from the
+#'  var.name column of 'data(output_name)'.
+#'  An example of such a data frame can be found with 'data(obs_temp)'
+#'
+#' @param sim.start,sim.end the start and end dates of the simulation period
+#' of the DYRESM-CAEDYM model run of interest.
+#' The date format must be "\%Y-\%m-\%d".
+#'
+#' @param plot.start,plot.end the start and end dates of the period to be
+#'  plotted in the format of "\%Y-\%m-\%d".
+#'
+#' @param xlabel the x axis label of the profile figure.
+#'
+#' @param min.depth,max.depth,by.value minimum and maximum depths in the profile
+#'  plot at an increment of by.value.
 #'
 #' @import dplyr
 #' @import ggplot2
 #'
-#' @return This function returns a ggplot object that can be modified with ggplot package functions.
+#' @return This function returns a ggplot object that can be modified
+#' with ggplot package functions.
 #'
 #' @examples
-#'  var.values<-ext_output(dycd.output=system.file("extdata", "dysim.nc", package = "dycdtools"),
+#'  var.values<-ext_output(dycd.output=system.file("extdata", "dysim.nc",
+#'                                                  package = "dycdtools"),
 #'                        var.extract=c("TEMP"))
 #'
 #'  for(i in 1:length(var.values)){
@@ -26,8 +42,8 @@
 #'
 #' # interpolate temperature for depths from 0 to 13 m at increment of 0.5 m
 #'   temp.interpolated<-interpol(layerHeights = dyresmLAYER_HTS_Var,
-#'                              var = dyresmTEMPTURE_Var,
-#'                              min.dept = 0,max.dept = 13,by.value = 0.5)
+#'                                var = dyresmTEMPTURE_Var,
+#'                                min.dept = 0, max.dept = 13, by.value = 0.5)
 #'
 #'  data(obs_temp)
 #' # profile plot of temperature sim and obs
@@ -38,7 +54,7 @@
 #'             plot.start="2017-06-06",
 #'             plot.end="2017-06-15",
 #'             xlabel = "Temperature \u00B0C",
-#'             min.depth = 0,max.depth = 13,by.value = 0.5)
+#'             min.depth = 0, max.depth = 13, by.value = 0.5)
 #'  p
 #'
 #' @export
@@ -57,8 +73,18 @@ plot_prof<-function(sim,
   #---
   # 1. simulation period
   #---
-  sim.date<-seq.Date(from = as.Date(sim.start,format="%Y-%m-%d"),
-                     to = as.Date(sim.end,format="%Y-%m-%d"),
+
+  if(any(is.na(ymd(plot.start, quiet = TRUE)),
+         is.na(ymd(plot.end, quiet = TRUE)),
+         is.na(ymd(sim.start, quiet = TRUE)),
+         is.na(ymd(sim.end, quiet = TRUE)))){
+
+    stop('Make sure date format is \'%Y-%m-%d\'\n')
+
+  }
+
+  sim.date <- seq.Date(from = as.Date(sim.start, format = "%Y-%m-%d"),
+                     to = as.Date(sim.end, format = "%Y-%m-%d"),
                      by="day")
 
   #---
